/********************************************************
IAR+STM8S_StdPeriph_Lib
Chip type: STM8S003k3 (16MHz HSE)
Project: 12VDC>220/110VAC sinewave converter
       : with simplified drivers
Version: 0
*********************************************************/
#include "stm8s.h"
#include "iostm003.c"
#include <stdbool.h>
#include "bitset.c"
#include "defines.c"
#include "display_work.c"
#include "periph_init.c"
#include <math.h>
#include "sw_calc.c"
#include "kwork.c"
#include "adc_work.c"



volatile u16 tim0, //display
             tim1, //delay timer
             tim2; //kb.timer
         
volatile bool restart_bit=1;
volatile bool iwdg_t1,iwdg_t2;


void devcon();

extern "C"
{
 INTERRUPT_HANDLER(TIM1_X3, 11) 
 {
  static u16 mst=0;
  u16 tmp_table;

  static u8 i=0;
  static bool rck=0, dir; 

 //adc control section 
  static u16 adc_cps=0;
  static u8 adc_cnt=0;
  
  u8 adc_tmp = ADC1->DRH;

  if((ADC1->CSR&0x0f)==3)
  {
   static u8 cs_trg=0;
   
   adc3val = adc_tmp;
   if(adc_tmp<0xff/10){TIM1->BKR&=~TIM1_BKR_MOE;cs_trg=0;}
   else if(adc_tmp<cs_ltrs)
   {
    cs_trg++;
    if(cs_trg>1)
    {
     cs_trg=0;
     TIM1->BKR&=~TIM1_BKR_MOE;
    }
   }
   else if(TIM1->BKR&TIM1_BKR_MOE)cs_trg=0;
 
   
   ADC1->CSR = 2;
  }
  else
  {
   if(mst<0xff/2);
   //else if(adc_tmp>(u8)(700UL*0xff/adc_ub)) mst=adc_uvh/adc_tmp;
   else if(adc_tmp>(u8)(700UL*0xff/1500)) mst=adc_uvh/adc_tmp;
   else mst=0x1ff;
   
   adc_cps+= adc_tmp;
   adc_cnt++;
   if(!adc_cnt)
   {
    adc2val = adc_cps>>8;
    adc_cnt = adc_cps = 0;
    adc_ready = 1;
   } 
   
   ADC1->CSR = 3;
  }
 //end of adc control section 



 /*ultrasonic bridge control section */
  if(restart_bit){restart_bit=0;mst=0;i=0;TIM1->BKR|=TIM1_BKR_MOE;}  

  tmp_table=(u16)((u32)s_table [i]*mst>>8);
  if(tmp_table>pwm_top){tmp_table=pwm_top;}

  if(!i)
  {
   dir=1;i=1;
   if(mst<0xff/2)mst+=5;
   if(rck) 
   { 
    rck=0;  
    TIM1->CCMR1 = TIM1_OCMODE_PWM1|TIM1_CCMR_OCxPE;
    TIM1->CCMR2 = 0x40; 
   } 
   else 
   { 
    rck=1;  
    TIM1->CCMR1 = 0x40;  
    TIM1->CCMR2 = TIM1_OCMODE_PWM1|TIM1_CCMR_OCxPE; 
   } 
   
   TIM1->EGR|=TIM1_EGR_COMG; 
  }
  else if((dir)&&(i<s_step/2-1))
  {
   TIM1->CCR1H=TIM1->CCR2H=tmp_table>>8;
   TIM1->CCR1L=TIM1->CCR2L=(u8)tmp_table;
   i++;
  }
  else
  {
   dir=0; i--;
   TIM1->CCR1H=TIM1->CCR2H=tmp_table>>8;
   TIM1->CCR1L=TIM1->CCR2L=(u8)tmp_table;
  }
 /*end of ultrasonic bridge control section */
  PA_ODR_ODR3^=1;
  iwdg_t1=1;
  TIM1->SR1 = 0;
 } 

 INTERRUPT_HANDLER(TIM2_OVF, 13)
 {
  display_data();
  kb.scan_keys();

  if(tim0)tim0--;
  if(tim1)tim1--;
  if(tim2)tim2--;
  
  //PA_ODR_ODR3^=1;

  iwdg_t2   = 1;
  TIM2->SR1 = 0;
 } 
 
 INTERRUPT_HANDLER(CLK_CSSR, 2)
 {
  TIM1->BKR &=~TIM1_BKR_MOE;
  while(1);  
 }

}

void main()
{
 cfg.gpio();
 cfg.ee();
 if(cfg.clk()==1)set_inval(1);
 else set_inval(0);
 cfg.adc(); 
 cfg.t1(pwm_top);
 cfg.t2();

 detect_7seg_polarity();
 calculate_sw_tb();
 ITC_SetSoftwarePriority(ITC_IRQ_CLK, ITC_PRIORITYLEVEL_2); 
 ITC_SetSoftwarePriority(ITC_IRQ_TIM1_OVF, ITC_PRIORITYLEVEL_3);
 ITC_SetSoftwarePriority(ITC_IRQ_TIM2_OVF, ITC_PRIORITYLEVEL_0); 

#ifndef debug 
 cfg.iwdg();
#endif

 asm("rim");

m:
 {

  devcon();

  if(iwdg_t1&&iwdg_t2){iwdg_t1=0;iwdg_t2=0;IWDG_ReloadCounter();}
 }
goto m;
}


void devcon()
{
 static bool itr=0,btn_trg;
 static enum {start,work,off,ocp,lobt} dev_st=start;
 
 if(kb.btn[0]==kb.rls && kb.btn[1]==kb.rls)btn_trg=0;

 switch (dev_st)
 {
  case start:
  {
   if(!itr)
   {
    itr=1;
    adc_ready=0;
    tim1=500;
    TIM1->BKR&=~TIM1_BKR_MOE;
   }
   
   static bool trg=0; 
   
   if(!tim1)
   {
    if(!trg)
    {
     trg=1;
     tim1=500;
     dbuf[2]='r';dbuf[1]='u';dbuf[0]='n'; dpp=5;
    }
    else
    {
     if(adc_ready)
     {
      static u8 lb_cnt=0;

      lb_cnt++;     
      if((u32)adc2val*adc_ub/0xff>eep.LoBt_trs){dev_st=work;lb_cnt=0;}
      else 
      {
       if(!lb_cnt) dev_st=lobt;
       adc_ready=0;
      }
     }     
    }
   }
   
   if(dev_st!=start){itr=0;trg=0;}
  }
  break;

  case work:
  {
   if(!itr)
   {
    itr=1;
    adc_ready=0;
    restart_bit=1;
   }
   
   if(adc_ready)
   {
    adc_ready=0;
    if(!(TIM1->BKR&TIM1_BKR_MOE) && !restart_bit)dev_st=ocp;
    
    u16 vtmp = (u32)adc2val*adc_ub/0xff;
    if(!tim0)
    {
     tim0=500;
     convert_data(vtmp/10); dpp=1;
    }    
    if(vtmp<800)dev_st=lobt;
    else if(vtmp>eep.LoBt_trs)tim1=2500;
    else if(vtmp<eep.LoBt_trs && !tim1)dev_st=lobt;    
   }
   
   if(kb.btn[0]==kb.hold && !btn_trg){btn_trg=1;dev_st=off;}
   

   if(dev_st!=work){itr=0;}
  }
  break;

  case off:
  {
   if(!itr)
   {
    itr=1;
    TIM1->BKR&=~TIM1_BKR_MOE;
    dbuf[2]='_';dbuf[1]='_';dbuf[0]='_'; dpp=5;
   }
   
   if(kb.btn[0]==kb.hold && !btn_trg){btn_trg=1;dev_st=start;}
 

   if(dev_st!=off){itr=0;}
  }
  break;

  case ocp:
  {
   if(!itr)
   {
    itr=1;
    dbuf[2]='O';dbuf[1]='C';dbuf[0]='P'; dpp=5;   
   }
   
   if(kb.btn[0]==kb.hold && !btn_trg){btn_trg=1;dev_st=start;}
   

   if(dev_st!=ocp){itr=0;}
  }
  break;  

  case lobt:
  {
   if(!itr)
   {
    itr=1;
    TIM1->BKR&=~TIM1_BKR_MOE;
    dbuf[2]='L';dbuf[1]='o';dbuf[0]='b'; dpp=1;
   }
   
   if((kb.btn[0]==kb.hold||kb.btn[0]==kb.prs) && !btn_trg)
   {
    btn_trg=1;
    if((u32)adc2val*adc_ub/0xff>eep.LoBt_trs)dev_st=start;
    else
    {
     tim1=1000;
     convert_data((u32)adc2val*adc_ub/0xff/10);
     dpp=1;
    }    
   }

   if(!tim1){dbuf[2]='L';dbuf[1]='o';dbuf[0]='b'; dpp=1;}  
   

   if(dev_st!=lobt){itr=0;}
  }
  break; 
  
  default:break;
 }

}

